// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;

namespace Smdn.Net.Imap4.Protocol.Client {
  // Layers:
  //   Session.ImapSession
  //   Transaction.IImapTransaction
  // * Protocol.ImapConnection/Protocol.ImapCommand/Protocol.ImapResponse
  //   (IMAP4)

  // handles '6. Client Commands'
  public sealed class ImapCommand {
    public string Tag {
      get { return tag; }
    }

    public string CommandString {
      get { return commandString; }
    }

    public ImapString[] Arguments {
      get { return arguments; }
    }

    public ImapCommand(string commandString)
      : this(commandString, string.Empty, EmptyArray<ImapString>.Instance)
    {
    }

    public ImapCommand(string commandString, string tag, params ImapString[] arguments)
    {
      this.commandString = commandString;
      this.tag = tag;
      this.arguments = arguments ?? EmptyArray<ImapString>.Instance;
    }

    public override string ToString()
    {
      return string.Format("{{CommandString={0}, Tag={1}, Arguments={2}}}",
                           commandString,
                           tag,
                           string.Join(", ", Array.ConvertAll(arguments, delegate(ImapString arg) { return arg.ToString(); })));
    }

    private /*readonly*/ string commandString;
    private /*readonly*/ string tag;
    private /*readonly*/ ImapString[] arguments;
  }
}