//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2008-2014 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

using Smdn.Collections;

namespace Smdn.Net.Imap4 {
  internal static class CollectionUtils {
#if NET_4_5
    internal static IReadOnlyList<T> ToNonNullReadOnly<T>(IReadOnlyList<T> list)
    {
      return list ?? EmptyArray<T>.Instance;
    }
#else
    internal static IList<T> ToNonNullReadOnly<T>(IList<T> list)
    {
      if (list == null)
        return new ReadOnlyCollection<T>(EmptyArray<T>.Instance);
      else if (list.IsReadOnly)
        return list;
      else
        return new List<T>(list).AsReadOnly();
    }
#endif

    private static class ReadOnlyDictionary<TKey, TValue> {
#if NET_4_5
      public static readonly IReadOnlyDictionary<TKey, TValue> Empty = new System.Collections.ObjectModel.ReadOnlyDictionary<TKey, TValue>(new Dictionary<TKey, TValue>(1));
#else
      public static readonly IDictionary<TKey, TValue> Empty = (new Dictionary<TKey, TValue>(1)).AsReadOnly();
#endif
    }

#if NET_4_5
    internal static IReadOnlyDictionary<TKey, TValue> ToNonNullReadOnly<TKey, TValue>(IReadOnlyDictionary<TKey, TValue> dictionary)
    {
      return dictionary ?? ReadOnlyDictionary<TKey, TValue>.Empty;
    }
#else
    internal static IDictionary<TKey, TValue> ToNonNullReadOnly<TKey, TValue>(IDictionary<TKey, TValue> dictionary)
    {
      if (dictionary == null)
        return ReadOnlyDictionary<TKey, TValue>.Empty;
      else if (dictionary.IsReadOnly)
        return dictionary;
      else
        return dictionary.AsReadOnly();
    }
#endif

    internal static string Join<T>(string separator, IEnumerable<T> values)
    {
#if NET_4_0
      return string.Join(separator, values.Select(val => val == null ? string.Empty : val.ToString()));
#else
      var b = new StringBuilder();
      var count = 0;

      foreach (var val in values) {
        if (count++ != 0)
          b.Append(separator);

        b.Append(val == null ? string.Empty : val.ToString());
      }

      return b.ToString();
#endif
    }

    internal static string Join<TKey, TValue>(string separator, IEnumerable<KeyValuePair<TKey, TValue>> pairs)
    {
#if NET_4_0
      return string.Join(separator, pairs.Select(pair => string.Concat("{", pair.Key, " => ", pair.Value, "}")));
#else
      var b = new StringBuilder();
      var count = 0;

      foreach (var pair in pairs) {
        if (count++ != 0)
          b.Append(separator);

        b.AppendFormat("{{{0} => {1}}}", pair.Key, pair.Value);
      }

      return b.ToString();
#endif
    }
  }
}

