// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Net.Imap4.Protocol;

namespace Smdn.Net.Imap4 {
  public class ImapExtendedSinglePartBodyStructure : ImapSinglePartBodyStructure, IImapBodyStructureExtension {
    public string MD5 {
      get; private set;
    }

    public ImapBodyDisposition Disposition {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyList<string>
#else
    public IList<string>
#endif
    Languages {
      get; private set;
    }

    public Uri Location {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyList<ImapData>
#else
    public IList<ImapData>
#endif
    Extensions {
      get; private set;
    }

    public ImapExtendedSinglePartBodyStructure
    (
#if NET_4_5
      ImapSinglePartBodyStructure baseStructure,
      string md5,
      ImapBodyDisposition disposition,
      IReadOnlyList<string> languages,
      Uri location,
      IReadOnlyList<ImapData> extensions
#else
      ImapSinglePartBodyStructure baseStructure,
      string md5,
      ImapBodyDisposition disposition,
      IList<string> languages,
      Uri location,
      IList<ImapData> extensions
#endif
    )
      : base(baseStructure)
    {
      this.MD5 = md5;
      this.Disposition = disposition;
      this.Languages = CollectionUtils.ToNonNullReadOnly(languages);
      this.Location = location;
      this.Extensions = CollectionUtils.ToNonNullReadOnly(extensions);
    }

    public override string ToString()
    {
      return string.Format("{{MD5={0}, Disposition={1}, Languages={2}, Location={3}, Extensions={4}, BaseStructure={{{5}}}}}",
                           MD5,
                           Disposition,
                           CollectionUtils.Join(", ", Languages),
                           Location,
                           CollectionUtils.Join(", ", Extensions),
                           base.ToString());
    }
  }
}
