// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2014 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Smdn.Net.Imap4 {
  public sealed class ImapNamespace : ICloneable {
#if NET_4_5
    public IReadOnlyList<ImapNamespaceDesc>
#else
    public IList<ImapNamespaceDesc>
#endif
    PersonalNamespaces {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyList<ImapNamespaceDesc>
#else
    public IList<ImapNamespaceDesc>
#endif
    OtherUsersNamespaces {
      get; private set;
    }

#if NET_4_5
    public IReadOnlyList<ImapNamespaceDesc>
#else
    public IList<ImapNamespaceDesc>
#endif
    SharedNamespaces {
      get; private set;
    }

    public ImapNamespace()
      : this(null, null, null)
    {
    }

    public ImapNamespace
    (
#if NET_4_5
      IReadOnlyList<ImapNamespaceDesc> personalNamespaces,
      IReadOnlyList<ImapNamespaceDesc> otherUsersNamespaces,
      IReadOnlyList<ImapNamespaceDesc> sharedNamespaces
#else
      IList<ImapNamespaceDesc> personalNamespaces,
      IList<ImapNamespaceDesc> otherUsersNamespaces,
      IList<ImapNamespaceDesc> sharedNamespaces
#endif
    )
    {
      this.PersonalNamespaces   = CollectionUtils.ToNonNullReadOnly(personalNamespaces);
      this.OtherUsersNamespaces = CollectionUtils.ToNonNullReadOnly(otherUsersNamespaces);
      this.SharedNamespaces     = CollectionUtils.ToNonNullReadOnly(sharedNamespaces);
    }

    object ICloneable.Clone()
    {
      return this.Clone();
    }

    public ImapNamespace Clone()
    {
      return new ImapNamespace(PersonalNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList(),
                               OtherUsersNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList(),
                               SharedNamespaces.Select((Func<ImapNamespaceDesc, ImapNamespaceDesc>)CloneNamespaceDesc).ToList());
    }

    private static ImapNamespaceDesc CloneNamespaceDesc(ImapNamespaceDesc desc)
    {
      return desc.Clone();
    }

    public override string ToString()
    {
      return string.Format("{{PersonalNamespaces={{{0}}}, OtherUsersNamespaces={{{1}}}, SharedNamespaces={{{2}}}}}",
                           CollectionUtils.Join(", ", PersonalNamespaces),
                           CollectionUtils.Join(", ", OtherUsersNamespaces),
                           CollectionUtils.Join(", ", SharedNamespaces));
    }
  }
}
